# Title:        MT-LJPost
# Summary:      A plugin for posting to livejournal whenever you post/edit an entry to MOveable Type
# Author:       Gavin Mogan (halkeye@halkeye.net) (http://www.halkeye.net/)
# Version:      1.6
# Date: April 03, 2004
#
# History
# 1.0.0 - January 29, 2004
#  - Inital Release
# 1.0.1 - January 30, 2004
#  - Removing needing so many variables
#  - Adding selecting of which blog you want.
#
# 1.0.2 - January 30, 2004
#  - Added config file because I got tired of always having to change the file for release
#      this file goes into your mt directory, not plugins (for now, i'll fix it next version or something)
#  # Format: blogid site username password
#  #         3 www.livejournal.com test test
#  #         4 www.deadjournal.com:90 test test
#  - Added handler to check for LJ::Simple and give a message without it
#
# 1.0.3 - January 30, 2004
#  - Entries will now edit after the inital post.
#  - If an entry doesn't save to livejournal, just hit save again
#  - Added support for exerpts / lj-cut
#  - Date of LJ post is date of Actual Entry. You'll have to watch out for backdated entries
#      If its a problem, set the date to now, post, then reset to old.
#      Temp Fix
#  - Convert Line Breaks actually working.
#
# 1.0.4 - January 30, 2004
#  - Date fix, oooopsies
#
# 1.0.5 - Febuary 09, 2004
#  - Started some hooks for MT Plugin Manager
#  - Cleaned up some error messages
#  - Disabled comments
#  - Deletes from livejournal when entry is deleted from MT
#  - Web interface to do configuration /mt-ljpost.cgi
#
# 1.6 - April 03, 2004
#  - Changed version schema
#  - Fixed problem when lj times out but data is still stored so you can no longer edit old entries.
#  - Added Template support (for entries, subjects are still not)
#  - MANY BUG fixes
#
# Information about this plugin can be found at
# http://www.halkeye.net/archives/000073.html (perm link later)
#
# Requires the LJ::Simple perl module - http://www.bpfh.net/computing/software/LJ%3a%3aSimple/
# Or CPAN as i prefer.
#
# Copyright 2004 Gavin Mogan
# This code cannot be redistributed without
# permission from the author. 


use lib('../../../extlib');
use lib('../../../lib');

my $ljtag = eval { require MT::Plugins::MTLJTag; 1 } ? 1 : 0;

use strict;
use warnings;

use MT::App;
use MT::ConfigMgr;
use MT::Blog;
use MT::PluginData;
use Digest::MD5 qw(md5_hex);

package MT::Plugins::MTLJPost;
use vars qw( $VERSION );
my $VERSION = "1.6";


@MT::Plugins::MTLJPost::ISA = qw( MT::App );

#----------------------------------------------------------------------------
# 
#   Name: init
#
#   Overview:
#       Sets up MT::App keys.
#
#   Calling Sequence ( and options )
#       N/A
#
#   Returns: MT::App object
#
#----------------------------------------------------------------------------

sub init {
        my $app = shift;
        $app->SUPER::init(@_) or return;
        $app->add_methods (
                'list'  => \&lj_post_show,
                'mtljpost_test' => \&lj_post_cfg,
                'mtljpost_cfg' => \&lj_post_cfg,
                'mtljpost_manage' => \&lj_post_manage,
                'mtljpost_manager' => \&lj_post_manage,
                'list_blogs' => \&list_blogs,
                );
        $app->{default_mode} = 'list';
        $app->{template_dir} = 'mtljpost';
        $app->{requires_login} = 1;
        $app->{user_class} = "MT::Author";

        $app;
}

sub get_entry_data($$)
{
        my $blog_id = shift;
        my $entry_id = shift;

        my %data;
        my $plugin = MT::PluginData->load({ plugin => 'MTLJPost',
                        key    => $blog_id . "_" . $entry_id }) || undef;
        return undef unless defined $plugin;
        my $ref = ref($plugin->data);
        if ($ref eq 'SCALAR') {
                my $itemid = $plugin->data;
                $data{'itemid'} = $$itemid;
        }
        else {
                %data = %{$plugin->data};
        }       
        $data{'itemid'} ||= -2;
        $data{'lastupdated'} = 'Never';;
        return \%data;
}

sub delete_entry_data($$)
{
        my $blog_id = shift;
        my $entry_id = shift;

        my %data;
        my $plugin = MT::PluginData->load({ plugin => 'MTLJPost',
                        key    => $blog_id . "_" . $entry_id }) || undef;
        return undef unless defined $plugin;
        $plugin->remove();
        return 1;
}


sub save_entry_data($$$)
{
        my $blog_id = shift;
        my $entry_id = shift;
        my $data = shift;

        my $plugin = MT::PluginData->load({ plugin => 'MTLJPost',
                        key    => $blog_id . '_' . $entry_id }) || undef;
        
        $plugin = MT::PluginData->new unless (defined $plugin);
        $plugin->plugin('MTLJPost');
        $plugin->key($blog_id . '_' . $entry_id);
        $plugin->data($data);
        $plugin->save or die $data->errstr;
}

sub get_login($)
{
        my $blog_id = shift;
        my $data = MT::PluginData->load({ plugin => 'MTLJPost', 
                                          key    => $blog_id }) || undef;
                                          
        my $logindata;
	$logindata = $data->data if defined $data;
	$logindata = {} unless defined $data;
        my %login = ();
#        return undef if ($logindata->{'enabled'} eq 0);
        $login{'site'} = $logindata->{'site'} || 'www.livejournal.com';
        $login{'username'} = $logindata->{'username'} || 'test';
        $login{'password'} = $logindata->{'password'} || 'test';
        $login{'enabled'} = $logindata->{'enabled'} || 0;
        $login{'site_subject'} = $logindata->{'site_subject'} || 0;
        $login{'format'} = $logindata->{'format'} || "%ENTRY%\n<br>\n[<a href=\"%SITE_URL%\">%BLOG_NAME%</a>] \n(<a href=\"%PERMALINK%\">Permanent link to this entry</a>)\n<br />\n";
        # This blog post
        return \%login;
}

sub lj_post_show {
        my $app = shift;
        $app->lj_post_cfg();    
}

sub lj_post_manage {
        require MT::Entry;
        my $app = shift;
        my $blog = undef;
        my %arg = ('sort' => 'created_on', direction => 'descend', limit=>25);
        my %param;
        my @entries;

        if ( defined $app->{query}->param('blog_id') ) {
                $blog = MT::Blog->load($app->{query}->param('blog_id')+0) || undef;
        }
        return $app->error ("No blog") unless defined $blog;

        $param{'mtljpost_version'} = $VERSION;
        my $iter = MT::Entry->load_iter({ blog_id => $blog->id,
                                          status => MT::Entry::RELEASE(),
                                        }, 
                                        'sort' => 'created_on',
                                        'direction' => 'descend',
                                        limit=>25,
                                        offset=>12,
                                        );
        while (my $entry = $iter->()) {
                my $data = get_entry_data($entry->blog_id, $entry->id) || next;
                push @entries, { id=>$entry->id, 'lastupdated'=>$data->{'lastupdated'}, };
        }
        $param{'entries'} = \@entries;
        $app->build_page ('MTLJPost_manager.tmpl', \%param);
}

sub lj_post_cfg {
        my $app = shift;
        my $blog = undef;
        my %param;
        if ( defined $app->{query}->param('blog_id') ) {
                $blog = MT::Blog->load($app->{query}->param('blog_id')+0) || undef;
        }
        
        $param{'mtljpost_version'} = $VERSION;
        if (defined $blog) {
                my $logindata = get_login($blog->id);
                
                $param{'blog_id'} = $blog->id;
		$param{'site'} = $logindata->{'site'};
		$param{'username'} = $logindata->{'username'};
		$param{'password'} = $logindata->{'password'};
		$param{'format'} = $logindata->{'format'};
		$param{'enabled'} = $logindata->{'enabled'};
		$param{'site_subject'} = $logindata->{'site_subject'};
		$param{'site_subject'} ||= 1;
                
                if (defined $app->{query}->param('__type'))
                {
			$param{'site'} = $app->{query}->param('site') if defined  $app->{query}->param('site');
			$param{'username'} = $app->{query}->param('username') if defined $app->{query}->param('username');
			$param{'password'} = $app->{query}->param('password') if defined $app->{query}->param('password');
			$param{'format'} = $app->{query}->param('format') if defined $app->{query}->param('format');
			$param{'enabled'} = $app->{query}->param('enabled') || 0;
			$param{'site_subject'} = $app->{query}->param('site_subject') || 0;
                        if ($app->{query}->param('__type') eq "mtljpost_test")
                        {
                                my $lj = new LJ::Simple ({
                                        user    => $param{'username'},
                                        pass    => $param{'password'},
                                        site    => $param{'site'},
                                });
                        
                                $param{'errmsg'} = $LJ::Simple::error unless (defined $lj);
                                if (defined $lj )
                                {
                                        my $message = $lj->message() || "Success";
                                        $message =~ s/(http|ftp|https|telnet):\/\/(\S+)/<a href=\"$1\:\/\/$2\">$2<\/a>/g;
                                        $param{'loginmsg'} = $message;
                                }
                        }
                        my $data = MT::PluginData->load({ plugin => 'MTLJPost',
                                                 key    => $blog->id }) || undef;
                        $data = MT::PluginData->new unless (defined $data);
                        $logindata->{'enabled'} = $param{'enabled'};
                        $logindata->{'username'} = $param{'username'};
                        $logindata->{'password'} = $param{'password'};
                        $logindata->{'site'} = $param{'site'};
                        $logindata->{'format'} = $param{'format'};
                        $logindata->{'site_subject'} = $param{'site_subject'};
                        $data->plugin('MTLJPost');
                        $data->key($blog->id);
                        $data->data($logindata);
                        $data->save or ($param{'errmsg'} .= $data->errstr);
                }
                $app->build_page ('MTLJPost_login.tmpl', \%param);
        }
        else
        {
                require MT::Blog;
                require MT::Permission;
                require MT::Entry;
                require MT::Comment;
                my $author = $app->{author};
                my @perms = MT::Permission->load({ author_id => $author->id });
                my %perms = map { $_->blog_id => $_ } @perms;
                my @blogs = MT::Blog->load;
                my @data;
                my $i = 1;
                for my $b (@blogs) {
                        my $blog_id = $b->id;
                        my $perms = $perms{ $blog_id };
                        next unless $perms->can_edit_config;
                        my $row = { id => $b->id, name => uc($b->name),
                            description => $b->description };
                        $row->{is_odd} = $i++ % 2 ? 1 : 0;
                        push @data, $row;
                }
                $param{blog_loop} = \@data;
                $app->build_page ('MTLJPost_index.tmpl', \%param);
        }
}

# I wish i could remember what this did :D
sub check_pm {
  my $app = shift;

}  

sub list_blogs {
    my $app = shift;
    my $q = $app->{query};
    require MT::Blog;
    require MT::Permission;
    require MT::Entry;
    require MT::Comment;
    my $author = $app->{author};
    my @perms = MT::Permission->load({ author_id => $author->id });
    my %perms = map { $_->blog_id => $_ } @perms;
    my @blogs = MT::Blog->load;
    my @data;
    my %param;
    my $i = 1;
    for my $blog (@blogs) {
        my $blog_id = $blog->id;
        my $perms = $perms{ $blog_id };
        next unless $perms && $perms->role_mask;
        next unless $perms->can_edit_config;
        my $row = { id => $blog->id, name => uc($blog->name),
                    description => $blog->description };
        $row->{is_odd} = $i++ % 2 ? 1 : 0;
        push @data, $row;
    }
    $param{blog_loop} = \@data;
    $app->build_page('list_blog.tmpl', \%param);
}

sub save_entry {
        my $entry = shift;
        my $logindata = MT::Plugins::MTLJPost::get_login($entry->blog_id);
        return 1 unless (defined $logindata and $logindata->{'enabled'});
       
	my $lj = new LJ::Simple ({
                user    => $logindata->{'username'},
                pass    => $logindata->{'password'},
                site    => $logindata->{'site'},
        });
        
        return $entry->error("Failed to log into LiveJournal:" . $LJ::Simple::error) unless defined $lj;
        eval {require LJ::Simple};
        return $entry->error("Need LJ::Simple") if $@;
        eval {require POSIX};
        return $entry->error("Need POSIX") if $@;

        use LJ::Simple;
        use POSIX qw(mktime difftime);

        # disable to debug
        
        my $blog = MT::Blog->load($entry->blog_id);
        # Build up the subject
        my $subject = '';
	$subject .= $blog->name . ": " if ($logindata->{'site_subject'});
	$subject .= $entry->title;
        
        MT::Plugins::MTLJTag::tagify(0) if $ljtag;
        my $body = MT->apply_text_filters($entry->text, $entry->text_filters);
        if ( defined $entry->text_more && $entry->text_more ne "") { # $entry->text_more =~ /^\s*$/s ) {
                $body .= "<lj-cut>\n";
                $body .= MT->apply_text_filters($entry->text_more, $entry->text_filters);
                $body .= "</lj-cut>";
        }
        MT::Plugins::MTLJTag::tagify(1) if $ljtag;
        
        my $t = HTML::Template->new_scalar_ref(\$logindata->{'format'},
                        filter => sub { my $text_ref = shift; $$text_ref =~ s/\%(.*?)\%/<TMPL_VAR NAME="$1">/g; },
                        die_on_bad_params=>0) || return $entry->error("Template Error");;
        $t->param('ENTRY', $body);
        $t->param('SITE_URL', $blog->site_url);
        $t->param('BLOG_NAME', $blog->name);
        $t->param('PERMALINK', $entry->permalink);

        my $date = $entry->created_on;
        $date =~ /(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/;
        my $time_t = mktime($6, $5, $4, $3, $2-1, $1-1900) || die "error";

        my $event=();
        
        my $data = MT::Plugins::MTLJPost::get_entry_data($entry->blog_id, $entry->id);
        my %Event=();
        # yes this is code duplication
        if (defined $data)
        {
                my $itemid = $data->{'itemid'} || 0;
                (defined $lj->GetEntries(\%Event, undef, "one", $itemid )) || return $entry->error("Failed to get Previous Entry: " . $LJ::Simple::error . "\n");
                $event = (values %Event)[0] || undef;
                unless (defined $event) {
                        $data = undef;
                        MT::Plugins::MTLJPost::delete_entry_data($entry->blog_id, $entry->id);
                        $lj->NewEntry(\%Event) || return $entry->error("Failed to create new entry: " . $LJ::Simple::error . "\n");
                        $event = \%Event;
                        $lj->Setprop_backdate(\%Event, 1);
                        # 3600 Seconds gives you an hour window to post it properly, otherwise it gets marked as back dated and you have to manually fix it.
                        $lj->Setprop_backdate(\%Event, 0) if (difftime(time(), $time_t) < 3600);
                }
        }
        else
        {
                $lj->NewEntry(\%Event) || return $entry->error("Failed to create new entry: " . $LJ::Simple::error . "\n");
                $event = \%Event;
                $lj->Setprop_backdate(\%Event, 1);
                # 3600 Seconds gives you an hour window to post it properly, otherwise it gets marked as back dated and you have to manually fix it.
                $lj->Setprop_backdate(\%Event, 0) if (difftime(time(), $time_t) < 3600);
        }
        $lj->SetEntry($event,$t->output) || return $entry->error("Failed to set entry: " . $LJ::Simple::error . "\n");
        $lj->SetSubject($event,$subject) || return $entry->error("Failed to set subject: " . $LJ::Simple::error . "\n");
        $lj->Setprop_preformatted($event, 1) if ($entry->convert_breaks);
        $lj->Setprop_preformatted($event, 0) unless ($entry->convert_breaks);
        $lj->SetDate($event,$time_t) || return $entry->error("Failed to set date: " . $LJ::Simple::error . "\n");       
        unless (defined $data) {
                my ($item_id,$anum,$html_id) = $lj->PostEntry($event) || undef;
                (defined $item_id) or return $entry->error("$0: Failed to post journal entry: " . $LJ::Simple::error . "\n");
                my %entry_data;
                $entry_data{'itemid'} = $item_id;
                $entry_data{'lastupdated'} = localtime();
                MT::Plugins::MTLJPost::save_entry_data($entry->blog_id, $entry->id,\%entry_data);
        } 
        else {
                $lj->EditEntry($event) || return $entry->error("$0: Failed to edit entry: " . $LJ::Simple::error . "\n");
        }
} 
1;
