# Title:        MT-LJPost
# Summary:      A plugin for posting to livejournal whenever you post/edit an entry to MOveable Type
# Author:       Gavin Mogan (halkeye@halkeye.net) (http://www.halkeye.net/)
# Version:      1.6
#
use lib '../lib';
use lib '../extlib';

use warnings;
use strict;
use MT::Plugins::MTLJPost;
use MT::Entry;
use MT::Blog;
use MT;
use HTML::Template;

my $mt_entry_save_handler;
my $mt_entry_remove_handler;

{
        # Trap ugly redefinition warnings
        local $SIG{__WARN__} = sub {  }; 

        # Replace MT::Entry::save with our 
        # own subroutine entrySave() but save
        # the code in $mt_entry_save_handler.
        #
        $mt_entry_save_handler = \&MT::Entry::save;
        *MT::Entry::save = \&entrySave; 
        # Same with MT::entry:remove
        $mt_entry_remove_handler = \&MT::Entry::remove;
        *MT::Entry::remove = \&removeEntry; 
}  

sub removeEntry {
        my $entry = shift;
        my $logindata = MT::Plugins::MTLJPost::get_login($entry->blog_id);
        return &$mt_entry_remove_handler($entry) unless (defined $logindata and $logindata->{'enabled'});
        my $lj = new LJ::Simple ({
                user    => $logindata->{'username'},
                pass    => $logindata->{'password'},
                site    => $logindata->{'site'},
        });
        
        unless (defined $lj) {
                return $entry->error("Failed to log into LiveJournal:" . $LJ::Simple::error);
        }

        my $err = &$mt_entry_remove_handler($entry);
        return $err unless ($err);

        my $blog = MT::Blog->load($entry->blog_id);
        my $data = MT::Plugins::MTLJPost::get_entry_data($entry->blog_id, $entry->id);
        if (defined $data)
        {
                my $itemid = $data->{'itemid'};
                (defined $lj->DeleteEntry($itemid)) || return $entry->error("Failed to get Delete Entry: " . $LJ::Simple::error . "\n");
                MT::Plugins::MTLJPost::delete_entry_data($entry->blog_id, $entry->id);
        }

}

sub entrySave {
        my $entry = shift;
        my $err = &$mt_entry_save_handler($entry);
	return $err unless ($err);

	return MT::Plugins::MTLJPost::save_entry($entry);
} 
1;
